#!/usr/bin/perl


#  Copyright notice:  Copyright © 2001 The Regents of the University of
#  California.  All rights reserved.
#
#  Other notices:  This software and text  files are provided as an
#  educational resource for non-commercial use only.
#
#  1.  Redistributions of this software must retain the copyright notice
#  on this page, this list of conditions and the following disclaimer.
#
#  2.Software, computer files, books or other documentation that incorporates
#  all or part of this file must reproduce the copyright notice on this page,
#  this list of conditions and the following disclaimer in the software
#  and/or other materials provided with the distribution.
#
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS OF THE UNIVERSITY OF CALIFORNIA
#  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
#  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
#  CONTRIBUTORS TO THE WORK BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS, LOSS OR USE, DATA OR PROFITS, OR
#  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE), ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


use Tk;
use Tk::FileSelect;
use Tk::Dialog;
use Tk::Balloon;
use Cwd;
use make_quizzes;
use File::Which;


BEGIN {
$latex = which('latex');
$dvipdf = which('dvipdf');
$latex2html = which('latex2html');
}

my ($shuffle,$answer_key_on,$solutions_on,$num_versions,$out_file,$file_in,$rand_seed,$latex_on,$make_all,$make_html);
#use Symbol;

$options_file = '.options';

if (-e $options_file) {
   readOptionsFile($options_file);
} else {
   $num_versions = 4;
   $out_file = 'q_out';
   $rand_seed = 12345;
}


my $cwd = cwd() . '/';

my $main = new MainWindow;

$main->minsize(qw(400 300));
$main->title("quiz maker");

checkForExecutable($latex,"You don't seem to have latex installed. 

Bummer. 

You won't be able to do much except produce tex files, 
so I'd recommend installing latex if you want to use this program."
);

checkForExecutable($dvipdf,"You don't seem to have dvipdf installed. 

Bummer. 

You won't be able to produce a pdf output with the all option,
so I'd recommend installing dvipdf if you want to use this program."
);

#checkForExecutable($latex2html,"You don't seem to have latex2html installed. 
#That's OK but you won't be able to produce an html web page.
#);




$main->configure(-menu => my $menubar =$main->Menu);
$menubar->configure(-activebackground=>"royalblue");
my $file = $menubar->cascade(-label => '~File');
my $options = $menubar->cascade(-label => '~Options');
my $help = $menubar->cascade(-label => '~Help');#, -tearoff=>0);

$help->command (
	-label => "Help",
	-accelerator => "Ctrl-h",
	-command => \&openHelp
);

$help->separator;

$file->command(
	-label => "Open Quiz...",
	-accelerator => "Ctrl-o",
	-command => \&openQuiz
);


$file->separator;

$file->command(
	-label => "Save Options...",
	-accelerator => "Ctrl-s",
	-command => \&saveOptions
);

$file->separator;

$file->command(
	-label => "Read Options...",
	-accelerator => "Ctrl-r",
	-command => \&readOptions
);

$file->separator;



$file->command(
	-label => "Save+Quit",
	-accelerator => "Ctrl-x",
	-command => \&saveQuit
);

$file->separator;



$file->command(
	-label => "Quit",
	-accelerator => "Ctrl-q",
	-command => sub{exit}
);



$main->bind('<Control_L><o>' => \&openQuiz);
$main->bind('<Control_L><s>' => \&saveOptions);
$main->bind('<Control_L><r>' => \&readOptions);
$main->bind('<Control_L><q>' => sub{exit});
$main->bind('<Control_L><x>' => \&saveQuit);
$main->bind('<Control_L><t>' => \&openConsole);
$main->bind('<Control_L><h>' => \&openHelp);


$OptionFrame=$main->Frame();#-background=>"gray" ,-borderwidth=>"5", -relief => "raised");
$OptionFrame->grid(-row=>1,-column=>1,-columnspan=>2);
#$OptionFrame->pack(-side => "left", -padx=>5);

my $ShuffleButton=$OptionFrame->Checkbutton(-text =>"shuffle", -variable => \$shuffle, -offvalue => 0, -onvalue => 1);
my $AnswerKeyButton=$OptionFrame->Checkbutton(-text =>"answer key", -variable => \$answer_key_on, -offvalue => 0, -onvalue => 1);
my $SolutionsButton=$OptionFrame->Checkbutton(-text =>"include solutions", -variable => \$solutions_on, -offvalue => 0, -onvalue => 1);
my $LatexButton=$OptionFrame->Checkbutton(-text =>"latex individual solutions", -variable => \$latex_on, -offvalue => 0, -onvalue => 1);
my $MakeAllButton=$OptionFrame->Checkbutton(-text =>"make one big pdf", -variable => \$make_all, -offvalue => 0, -onvalue => 1);
my $MakeHtmlButton=$OptionFrame->Checkbutton(-text =>"make html", -variable => \$make_html, -offvalue => 0, -onvalue => 1, -command => \&checkExistsLatex2html);

#$AnswerKeyButton->deselect();
$ShuffleButton -> grid(-row=>0,-column=>0,-sticky=>'w');
$AnswerKeyButton -> grid(-row=>1,-column=>0,-sticky=>'w');
$SolutionsButton -> grid(-row=>2,-column=>0,-sticky=>'w');
$LatexButton -> grid(-row=>3,-column=>0,-sticky=>'w');
$MakeAllButton -> grid(-row=>4,-column=>0,-sticky=>'w');
$MakeHtmlButton -> grid(-row=>5,-column=>0,-sticky=>'w');

if (!defined($latex)) {
   $latex_on = 0;
   $LatexButton->configure(-state => 'disabled');
}

if (!defined($dvipdf)) {
   $make_all = 0;
   $LatexButton->configure(-state => 'disabled');
}

# my $LabelShuffle = $ShuffleButton -> Label(-text=>"Shuffle");
#$LabelShuffle -> grid(-row=>1,-column=>1);

#my $StatusLabel = $OptionFrame->Label(-text =>"label", -width => 10);
#$StatusLabel -> grid(-row=>0,-column=>0,-sticky=>'w');

#my $Balloon = $OptionFrame->Balloon(-statusbar => $StatusLabel);
my $Balloon = $OptionFrame->Balloon();


my $VersionLabel = $OptionFrame -> Label(-text=>"# of versions:");
my $VersionEntry = $OptionFrame -> Entry(-textvariable => \$num_versions, -width => 2);

$VersionLabel -> grid(-row=>0,-column=>1, -sticky => "e");
$VersionEntry -> grid(-row=>0,-column=>2, -sticky => "e");


#my $InputFileLabel = $OptionFrame -> Label(-text=>"   quiz file:");
my $InputFileButton = $OptionFrame -> Button(-text=>"   quiz file:", -highlightthickness => 0, -borderwidth => 0, -command => \&openQuiz);
my $InputFileEntry = $OptionFrame -> Entry(-textvariable => \$file_in, -width => 8, -justify => 'right');

$InputFileButton -> grid(-row=>1,-column=>1, -sticky => "e");
$InputFileEntry -> grid(-row=>1,-column=>2, -sticky => "e");
$InputFileEntry->xview(moveto => '1.0');


#my $OutputFileLabel = $OptionFrame -> Label(-text=>"   output file:");
my $OutputFileButton = $OptionFrame -> Button(-text=>"  output file:", -highlightthickness => 0, -borderwidth => 0, -command => \&selectOutputFile);
my $OutputFileEntry = $OptionFrame -> Entry(-textvariable => \$out_file, -width => 8, -justify => 'right');

#$OutputFileLabel -> grid(-row=>3,-column=>1, -sticky => "e");
$OutputFileButton -> grid(-row=>3,-column=>1, -sticky => "e");
$OutputFileEntry -> grid(-row=>3,-column=>2);
$OutputFileEntry->xview(moveto => '1.0');

#$OutputFileLabel -> bind ('<Double-ButtonPress-1>',  \&selectOutputFile());

my $RandSeedLabel = $OptionFrame -> Label(-text=>"random seed:");
my $RandSeedEntry = $OptionFrame -> Entry(-textvariable => \$rand_seed, -width => 8, -justify => 'right');

$RandSeedLabel -> grid(-row=>2,-column=>1);
$RandSeedEntry -> grid(-row=>2,-column=>2);

my $GoButton = $OptionFrame -> Button(-text => "Go", -command =>\&go_quizzes);

$GoButton -> grid(-row=>5,-column=>2,-sticky=>'w');

$options->command(-label => "Debug Console", -accelerator => "Ctrl-t", -command => \&openConsole);

#$Balloon->attach($VersionLabel, -balloonmsg => "Balloon help message", -statusmsg => "Status bar message");
$Balloon->attach($VersionLabel, -balloonmsg => "Create this many randomized versions");
#$Balloon->attach($OutputFileLabel, -balloonmsg => "e.g. 'q' will produce q1.tex, q2.tex ...");
$Balloon->attach($OutputFileButton, -balloonmsg => "e.g. 'q' will produce q1.tex, q2.tex. Click here to browse");
#$Balloon->attach($InputFileLabel, -balloonmsg => "The template. Go to file menu to browse instead");
$Balloon->attach($InputFileButton, -balloonmsg => "The template. Click here to browse instead");
$Balloon->attach($RandSeedLabel, -balloonmsg => "Change this to get different random values");
$Balloon->attach($ShuffleButton, -balloonmsg => "To randomize order of problems and choices"); 
$Balloon->attach($AnswerKeyButton, -balloonmsg => "Outputs multiple choice answers"); 
$Balloon->attach($SolutionsButton, -balloonmsg => "Outputs these, if they're provided "); 
$Balloon->attach($LatexButton, -balloonmsg => "Latex's individual versions separately "); 
$Balloon->attach($MakeAllButton, -balloonmsg => "Creates a big pdf of everything"); 
$Balloon->attach($MakeHtmlButton, -balloonmsg => "Creates html version of everything"); 
$Balloon->attach($GoButton, -balloonmsg => "See 'options->Debug Console' for possible errors"); 

if ($make_html == 1) {
   checkExistsLatex2html();
   $make_html = 0;
}

MainLoop;

sub openHelp {
   if (! Exists($HelpWidget)) {
      $HelpWidget = $main->Toplevel;
      $HelpWidget->minsize(qw(200 100));
      $HelpWidget->title('Help');
      $HelpWidget->Button(-text => 'Close', -command => sub {$HelpWidget->withdraw })->pack;
      $HelpText = $HelpWidget->Scrolled('Text');
      $HelpText->pack(-expand => 1, -fill => 'both'); 
      open(HELP,"<help.txt");
      @helpfile = <HELP>;
      close(HELP);
      $helpfile = join(' ',@helpfile);
      $HelpText->insert('end',$helpfile);

   }
   else {
      $Help->deiconify;
      $Help->raise;
   }
}

sub openConsole {
   if (! Exists($console)) {
      $console = $main->Toplevel;
      $console->minsize(qw(400 600));
      $console->title('Console');
      #open(TST,'>tst.d');
      $console->Button(-text => 'Close', -command => sub {
	         $console->withdraw; 
#		 close(STDOUT); 
#		 close(STDERR);
	         }
	      )->pack;
      $text = $console->Scrolled('Text');
      $text->pack(-expand => 1, -fill => 'both'); 
      eval {

      open(STDOUT,'>&STDOUT');
      open(STDERR,'>&STDERR');
      tie (*STDOUT, 'Tk::Text',$text); 
      tie (*STDERR, 'Tk::Text',$text); 

#      open($stdout,'>&STDOUT');
#      open($stderr,'>&STDERR');
#      tie (*$stdout, 'Tk::Text',$text); 
#      tie (*$stderr, 'Tk::Text',$text); 
   };
   $console->OnDestroy(sub {untie *STDOUT; untie *STDERR;});
#   print $stdout "ahhh...";
   }
   else {
      $console->deiconify;
      $console->raise;
   }
}

sub selectOutputFile{
my $types = [
     ['Text Files',       ['.tex', '.TEX']],
     ['XML Files',        ['.xml', '.XML']],
     ['All Files',        '*',             ],
 ];

$out_file = $main->getOpenFile(-filetypes=>$types, -title => "Select Output File (will ignore extension)");
$out_file =~ s/\.tex//i;
$out_file =~ s/\.xml//;
$out_file =~ s/$cwd//;
}


sub openQuiz {
my $types = [
     ['XML Files',        ['.xml', '.XML']],
     ['Text Files',       ['.txt', '.text']],
     ['All Files',        '*',             ],
 ];

$file_in = $main->getOpenFile(-filetypes=>$types, -title => "Open Quiz File");
$file_in =~ s/$cwd//;
}


sub saveOptions {

 $options_file = $main->getSaveFile(-title => "Save Options", -initialfile => ".options");
 
 open (OUT, ">$options_file") or die "Can't open $options_file";
 print OUT makeArguments(), "\n";
 close(OUT);

}

sub saveQuit {
   saveOptions();
   exit;
}

sub readOptionsFile {
   my $file_o = shift;
   open (IN, "<$file_o") or die "Can't open $file_o";
   my @command_line = <IN>;
   close(IN);
   $command_line = join(' ',@command_line);
   $hash =   parse_command_line($command_line);
   $shuffle = $hash->{shuffle};
   $answer_key_on = $hash->{answer_key_on}; 
   $solutions_on = $hash->{solutions_on};
   $num_versions = $hash->{num_versions};
   $out_file = $hash->{out_file};
   $file_in = $hash->{file_in};
   $rand_seed = $hash->{rand_seed};
   $latex_on = $hash->{latex_on};
   $make_all = $hash->{make_all};
   $make_html = $hash->{make_html};
}

sub readOptions {
   my $types = [
   ['Options Files',       ['.options']],
   ['All Files',        '*',             ],
   ];

   $options_file = $main->getOpenFile(-filetypes=>$types, -title => "Read Options File");
   readOptionsFile($options_file);
}

sub makeArguments {
   my $out='';
   die "you must specify an input file in the menu File->Open Quiz" if ($file_in !~ /\w/);
   die "you must specify an output file" if ($out_file !~ /\w/);
   die "you must specify an integer for random seed " if ($rand_seed !~ /\d/);
   die "you must specify an integer for the # of versions " if ($num_versions !~ /\d/);
   $out .= " -shuffle" if ($shuffle ==1);
   $out .= " -answer_key_on" if ($answer_key_on ==1);
   $out .= " -solutions on" if ($solutions_on == 1);
   $out .= " -num $num_versions";
   $out .= " -out $out_file";
   $out .= " -in $file_in";
   $out .= " -rand_seed $rand_seed";
   $out .= " -latex" if ($latex_on ==1);
   $out .= " -all" if ($make_all ==1);
   $out .= " -html" if ($make_html ==1);
   return $out;
}

sub go_quizzes {
   make_quizzes(makeArguments);
}

sub print_commands {
   print makeArguments, "\n";
}

sub checkForExecutable {

   my ($isItDefined,$Blurb) = @_;

   if (!defined($isItDefined)) {
      $ResponseButton = $main->DialogBox( -title   => "Missing software",
      -buttons => [ "Continue", "Cancel" ] );
      $ResponseButton->minsize(400,300);
      $ResponseButton->add("Label", -text => $Blurb)->pack();
      $response = $ResponseButton->Show();
      if ($response eq 'Cancel') {
	 exit;
      }
   }
}

sub checkExistsLatex2html {
   if (!defined($latex2html) && $make_html == 1) {
      #print "in turn off latex2html\n";

      $MakeHtmlButton->configure(-state => 'disabled');

      checkForExecutable($latex2html,"You don't seem to have latex2html installed. 
      That's OK but you won't be able to produce an html web page."
   );
}
}


