#!/usr/bin/perl

package make_quizzes;
use shuffle;
use IPC::Open3;

@ISA = ('Exporter');
@EXPORT = qw(&parse_command_line &make_quizzes &set_file_handles);




#  Copyright notice:  Copyright  2001 The Regents of the University of
#  California.  All rights reserved.
#
#  Other notices:  This software and text  files are provided as an
#  educational resource for non-commercial use only.
#
#  1.  Redistributions of this software must retain the copyright notice
#  on this page, this list of conditions and the following disclaimer.
#
#  2.Software, computer files, books or other documentation that incorporates
#  all or part of this file must reproduce the copyright notice on this page,
#  this list of conditions and the following disclaimer in the software
#  and/or other materials provided with the distribution.
#
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS OF THE UNIVERSITY OF CALIFORNIA
#  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
#  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
#  CONTRIBUTORS TO THE WORK BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS, LOSS OR USE, DATA OR PROFITS, OR
#  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE), ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

my ($read_file_handle,$write_file_handle,$error_file_handle);

sub make_quizzes {
   parse_command_line(shift);
   $preamble = '
   \documentclass{report}  
   \usepackage{color}
   \usepackage{epsfig}
   \pagecolor{white}
   \pagestyle{empty}
   \newcommand{\br}{{\bf r}}
   \advance\textheight by 0.7in
   \advance\topmargin by -0.4in
   \oddsidemargin -0.5in
   %\setlength{\parindent}{0in}
   \parindent 0in
   \textheight 10.0in
   \textwidth 7.0in
   \newcommand\degrees[1]{\ensuremath{#1^\circ}}
   \newcommand{\lt}{<}
   \newcommand{\gt}{>}
   \newcommand{\ampersand}{&}
   \begin{document}
   '
   ;

   $end = '
   \end{document}
   ';

   $newpage = '
   \newpage
   '
   ;


   srand $rand_seed;

   open (IN, "<$file_in") or die "Can't open $file_in";


   $y = '';
   $content = '';
   while (<IN>) {
      $content .= $_;
   }

   if ($content =~ /<problem>/) {
      $xml_format = 1;
   } else {
      $xml_format = 0;
   }
    
   die "Whoops, you need to have valid xml in your quiz file to be able to shuffle a quiz.
        Turn of shuffling, or check your quiz file to make sure it's valid xml.\n"
        if ($shuffle == 1 && $xml_format == 0);


	if ($shuffle == 0 && $xml_format == 1) {
	   "I'll assume you want to do what you said and so I won't shuffle the
	   quizzes.\n";
	   $no_shuffling = 1;
	   $shuffle = 1;
	} else {
	   $no_shuffling = 0;
	}

   make_dtd if ($shuffle == 1);

   $master_content = $content;

   $whole_shabang = '';

   for($i=1; $i <= $num_versions; $i++) {

      my ($newcontent, $ncontent, $y, $string); 
      $content =$master_content;

      #This line gives very non random behavior for the fourth call to rand
      # srand $rand_seed++;

      if ($num_versions > 1) {
	 $tex_name = "$out_file$i.tex";
      }
      else
      {
	 $tex_name = $out_file . '.tex';
      }
      open (OUT, ">$tex_name") or die "Can't open $out_file";
      push @texname, $tex_name;

      print OUT "%In make_quizzes\n";


      print OUT "%argument in: " , $input, "\n" ;


      print OUT "%file_in = ", $file_in, "\n" ;
      print OUT "%ran_seed = ", $ran_seed, "\n" ;



      while() {

	 if ($content !~ /<def>/s) { 
	    $newcontent .= $content;
	    last;
	 }

	 $content =~ s|(^.*?</def>)||sx;
	 $beginning = $1;
	 $beginning =~ s|<def>(.*)</def>||sx;
	 eval $1;
	 if ($@) {
	    numbered_print($1);
	    die $@;
	 }
	 $newcontent .= $beginning;
      }

      while() {

	 if ($newcontent !~ /<eval>/s) { 
	    $ncontent .= $newcontent;
	    last;
	 }

	 $newcontent =~ s|(^.*?</eval>)||sx;
	 $beginning = $1;
	 $beginning =~ s|<eval>(.*)</eval>||sx;
	 $ncontent .= $beginning;
	 $ncontent .= round(eval $1,3);
	 if ($@) {
	    die $@;
	 }
      }


      $ncontent =~ s/\@/ FuriouScAbbAGE/g;
      $ncontent =~ s/\\/ ReLatIveFrU1tFlY /g; #since it gets obliterated somehow due to quoting

      $string = '$y = sprintf ("%s", qq@';
      $string .= $ncontent;
      $string .= '@)';
      eval $string;
      if ($@) {
	 die $@;
      }

      $y =~ s/ ReLatIveFrU1tFlY /\\/g;
      $y =~ s/ FuriouScAbbAGE/\$/g;

      #change exponential e(blah) to latex 10^{blah} notation
      $y =~ s/(\.|\d)e(-?)0(\d)/$1\\times 10^{$2$3}/g;
      $y =~ s/(\.|\d)e(-?[1-9]\d)/$1\\times 10^{$2}/g;

      $y =~ s/(\.|\d)e\+0(\d)/$1\\times 10^{$2}/g;
      $y =~ s/(\.|\d)e\+([1-9]\d)/$1\\times 10^{$2}/g;

      #print $y;

      print OUT $preamble;
      print OUT "%The results:\n";
      ($y, my $answer_key) = shuffle_quiz($y,$solutions_on,$no_shuffling) if ($shuffle == 1);
      print OUT $y;
      print OUT $end;


      $y .= $newpage . $answer_key if ($answer_key_on == 1);
      $whole_shabang .= $y;
      $whole_shabang .= $newpage;


      close(OUT);
   }


   if ($latex_on == 1) {
      foreach $name (@texname) {
	 # open3($write_file_handle,$read_file_handle,$error_file_handle,"latex", $name) || die "can't latex $name\n";
	 #open3($stdout,$read_file_handle,$stderr,"latex", $name);
	 #close($stderr);
	 #close($stdout);
	 #close($read_file_handle);
	 system("latex $name");
      }
   }

   if ($make_all == 1) {
      $all_name = $out_file . "_all";
      $all_name_tex = $out_file . "_all" . '.tex';
      open (OUT, ">$all_name_tex") or die "Can't open $out_file_tex";
      print OUT $preamble;
      print OUT $whole_shabang;
      print OUT $end;
      close(OUT);
      system("latex $all_name");
      system("dvipdf $all_name");
   }

   if ($make_html == 1) {
      $html_name = $out_file . "_html" . '.tex';
      open (OUT, ">$html_name") or die "Can't open $out_file";
      print OUT $preamble;
      print OUT $whole_shabang;
      print OUT $end;
      close(OUT);
      system("latex $html_name");
      system("latex2html -split 0 $html_name");
   }

}

#sub round {
#   my($raw,$n) = @_;
#   if ($raw =~ /[a-z]/i) {
#      return $raw;
#   }
#   my $format = "%."."$n"."f";
#   return sprintf($format,$raw);
#}

sub round {
   my($raw,$n) = @_;
        return $raw unless $raw =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/;
   my $format = "%"."$n". "." . "$n" . "g";
   return sprintf($format,$raw);
}

sub ran_round {
   my ($max) = @_;
   my $raw = $max*rand;
   $raw *= 100;
   return 0.01*int($raw);
}

sub trunc {
   my $in = shift;
   return 0.01*int(100*$in);
}

sub ran_int {
   my ($max) = @_;
   my $raw = $max*rand;
   return int($raw);
}


sub ran_word {
   my (@words) = @_;
   my $index = int( scalar(@words) * rand);
   return $words[$index];
}

sub choices {
   my (@words) = @_;
   my $n = scalar(@words);
   my $i;
   my $n_a = 97;# integer corresponding to "a";
   die "too many items: $n > 26\n" if ($n > 26);
   my @order;
   for($i=0; $i < $n; $i++) {
      $order[$i] = $i;
   }
   for($i=0; $i < 2*$n; $i++) {
      my $m = ran_int($n);
      my $l = ran_int($n);
      next if ($l == $m);
      my $tmp = $order[$m];
      $order[$m] = $order[$l];
      $order[$l] = $tmp;
   }
   my $text = '\begin{description}'."\n";
   for($i=0; $i < $n; $i++) {
      $text .=  '\item[(' . chr($n_a+$i) . ')] ' . $words[$order[$i]] . "\n";
   }
   $text .= '\end{description}'."\n";
   return $text;
}

sub numbered_print {
   my @lines = split("\n",@_[0]);
   for (my $i=0; $i < scalar(@lines); $i++) {
      print $i+1,"\t",$lines[$i],"\n";
   }
}



sub parse_command_line {
   my $input = shift;

   $out_file = "version";

   $num_versions = 1;
   $shuffle = 0;
   $answer_key_on = 0;
   $solutions_on = 0;
   $latex_on = 0;
   $make_all = 0;
   $make_html = 0;

   if ($input =~ /-help/ix) {
      print "make_quizzes.pl -num {# of quizzes} -shuffle -solutions_on -answer_key_on -out {output file} -in {input file} -latex {latex the results} -html {latex and latex2html} -rand {rand seed}\n";
      exit;
   }

   if ($input =~ /-shuff/ix) {
      $shuffle=1;
   }

   if ($input =~ /-ans\w*?key/ix) {
      $answer_key_on=1;
   }


   if ($input =~ /-solut\w*?\s(.+?)(\s|$)/ix) {
      if ($1 eq 'on') {
	 $solutions_on = 1;
      } 
      elsif ($1 eq 'off') {
	 $solutions_on = 0;
      } else {
	 die "option -solutions must be -solutions on or -solutions off not $1";
      }
   }



   if ($input =~ /-num\w*?\s(.+?)(\s|$)/ix) {
      $num_versions = $1;
   }

   if ($input =~ /-out\s(.+?)(\s|$)/ix) {
      $out_file = $1;
   }

   if ($input =~ /-in\s(.+?)(\s|$)/ix) {
      $file_in = $1;
   }

   $rand_seed = 12345;
   if ($input =~ /-rand\w*\s(.+?)(\s|$)/i) {
      $rand_seed = $1;
   }
   if ($input =~ /-late/ix) { $latex_on = 1;}

   if ($input =~ /-all/ix) { $make_all = 1;}

   if ($input =~ /-htm/ix) { $make_html = 1;}

   return {
      shuffle => $shuffle,
      answer_key_on => $answer_key_on,
      solutions_on => $solutions_on,
      num_versions => $num_versions,
      out_file => $out_file,
      file_in => $file_in,
      rand_seed => $rand_seed,
      latex_on => $latex_on,
      make_all => $make_all,
      make_html => $make_html
   };
}

sub set_file_handles {
   $read_file_handle = shift;
   $write_file_handle = shift;
   $error_file_handle = shift;
}

1;
